@@ -1,7 +1,7 @@
-window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
+window.zoo <- function(x, index. = index(x), start = NULL, end = NULL, ...)
 {
-  all.indexes <- index.zoo(x)
-  in.index <- MATCH(all.indexes, index, nomatch = 0) > 0
+  all.indexes <- index(x)
+  in.index <- MATCH(all.indexes, index., nomatch = 0) > 0
 
   if(length(start) == 2 && !is.null(attr(x, "frequency")) && is.numeric(all.indexes)) {
     freq <- attr(x, "frequency")
@@ -14,7 +14,7 @@ window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
 
   if(is.null(start)) {
     if(is.null(end)) {
-      wi <- which(MATCH(all.indexes, index, nomatch = 0) > 0)
+      wi <- which(MATCH(all.indexes, index., nomatch = 0) > 0)
       return(x[wi,,])
     } else {
       wi <- which(in.index & all.indexes <= end)
@@ -30,10 +30,10 @@ window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
   }
 }
 
-"window<-.zoo" <- function(x, index = index.zoo(x), start = NULL, end = NULL, ..., value)
+"window<-.zoo" <- function(x, index. = index(x), start = NULL, end = NULL, ..., value)
 {
-  ix <- index.zoo(x)
-  stopifnot(all(MATCH(index, ix, nomatch = 0) > 0))
+  ix <- index(x)
+  stopifnot(all(MATCH(index., ix, nomatch = 0) > 0))
   
   if(length(start) == 2 && !is.null(attr(x, "frequency")) && is.numeric(ix)) {
     freq <- attr(x, "frequency")
@@ -44,10 +44,10 @@ window.zoo <- function(x, index = index.zoo(x), start = NULL, end = NULL, ...)
     end <- floor(end[1]*freq + (end[2] - 1) + .0001)/freq
   }
   
-  if (!is.null(start)) index <- index[index >= start]
-  if (!is.null(end)) index <- index[index <= end]
+  if (!is.null(start)) index. <- index.[index. >= start]
+  if (!is.null(end)) index. <- index.[index. <= end]
 
-  wi <- which(MATCH(ix, index, nomatch = 0) > 0)
+  wi <- which(MATCH(ix, index., nomatch = 0) > 0)
   if (length(dim(x)) == 0)
   	  x[wi] <- value
   else
@@ -69,10 +69,10 @@ lag.zoo <- function(x, k = 1, na.pad = FALSE, ...)
    if (k == 0) return(x)
    if (abs(k) > nr) k <- nr
    if (k > 0)  {
-	   xx <- x[-seq(1, length = k),,]
+	   xx <- x[-seq(1, length = k),, drop = FALSE]
 	   attr(xx, "index") <- index(x)[-seq(to = nr,length = k)]
    } else {
-	   xx <- x[-seq(to = nr, length = -k),,]
+	   xx <- x[-seq(to = nr, length = -k),, drop = FALSE]
 	   attr(xx, "index") <- index(x)[-seq(1, length = -k)]
    }
    if (na.pad) merge(zoo(,time(x)), xx, all = c(TRUE, FALSE)) else xx
@@ -104,11 +104,10 @@ lag.zooreg <- function(x, k = 1, na.pad = FALSE, ...)
 diff.zoo <- function(x, lag = 1, differences = 1, arithmetic = TRUE, na.pad = FALSE, ...)
 {
     ix <- index(x)
-    stopifnot(lag >= 1, differences >= 1)
+    stopifnot(differences >= 1)
     if (!arithmetic) x <- log(x)
-    for(i in 1:differences) {
-	x <- x - lag(x, k = -lag, ...)
-    }
+	if (lag > 0) for(i in 1:differences) x <- x - lag(x, k = -lag, ...)
+	else for(i in 1:differences) x <- lag(x, k = -lag, ...) - x
     if (!arithmetic) x <- exp(x)
     if (na.pad) merge(zoo(,ix), x, all = c(TRUE, FALSE)) else x
 }
