@@ -217,9 +217,11 @@ str.zoo <- function(object, ...)
   if(NCOL(object) > 0L & is.null(colnames(object))) stop("only possible for zoo series with column names")
   wi <- match(x, colnames(object))
   if(is.na(wi)) {
-    object <- cbind(object, value)
-    if(is.null(dim(object))) dim(object) <- c(length(object), 1)
-    colnames(object)[NCOL(object)] <- x  
+    if(!is.null(value)) {
+      object <- cbind(object, value)
+      if(is.null(dim(object))) dim(object) <- c(length(object), 1)
+      colnames(object)[NCOL(object)] <- x  
+    }
   } else {
     if(is.null(value)) {
       object <- object[, -wi, drop = FALSE]
@@ -326,8 +328,39 @@ median.zoo <- function(x, na.rm = FALSE)  median(coredata(x), na.rm = na.rm)
 
 quantile.zoo <- function(x, ...) quantile(coredata(x), ...)
 
-transform.zoo <- function(`_data`, ...) {
+transform.zoo <- function(`_data`, ...)
+{
+  ## zoo series elements
   if(is.null(dim(coredata(`_data`)))) warning("transform() is only useful for matrix-based zoo series")
-  zoo(transform.data.frame(data.frame(coredata(`_data`)), ...),
-    index(`_data`), attr(`_data`, "frequency"))
+  ix <- index(`_data`)
+  freq <- attr(`_data`, "frequency")
+  `_data` <- as.data.frame(`_data`)
+
+  ## get transformations
+  e <- eval(substitute(list(...)), `_data`, parent.frame())
+  trafo <- names(e)
+  inx <- match(trafo, names(`_data`))
+  matched <- !is.na(inx)
+
+  ## evaluate transformations
+  if(any(matched)) {
+    `_data`[inx[matched]] <- e[matched]
+    `_data` <- data.frame(`_data`)
+  }
+  if (!all(matched)) 
+    `_data` <- do.call("data.frame", c(list(`_data`), e[!matched]))
+
+  ## return zoo
+  zoo(`_data`, ix, freq)
+}
+
+`dim<-.zoo` <- function(x, value) {
+  d <- dim(x)
+  l <- length(x)
+  ok <- isTRUE(all.equal(d, value)) ||                                  ## no change
+    (is.null(d) && l == 0L && all(value == c(length(index(x)), 0L))) || ## zero-length vector -> 0-column matrix
+    (is.null(d) && l > 0L && all(value == c(l, 1L))) ||                 ## positive-length vector -> 1-column matrix
+    (!is.null(d) && d[2L] <= 1L && is.null(value))                      ## 0- or 1-column matrix -> vector
+  if(!ok) warning("setting this dimension may lead to an invalid zoo object")
+  NextMethod()
 }
